function N = getN(om, selector, name)
%GETN  Returns the number of variables, constraints or cost rows.
%   N = GETN(OM, SELECTOR)
%   N = GETN(OM, SELECTOR, NAME)
%
%   Returns either the total number of variables/constraints/cost rows
%   or the number corresponding to a specified named block.
%
%   Examples:
%       N = getN(om, 'var')         : total number of variables
%       N = getN(om, 'lin')         : total number of linear constraints
%       N = getN(om, 'nln')         : total number of non-linear constraints
%       N = getN(om, 'cost')        : total number of cost rows (in N)
%       N = getN(om, 'var', name)   : number of variables in named set
%       N = getN(om, 'lin', name)   : number of linear constraints in named set
%       N = getN(om, 'nln', name)   : number of non-linear cons. in named set
%       N = getN(om, 'cost', name)  : number of cost rows (in N) in named set
%
%   See also OPF_MODEL.

%   MATPOWER
%   $Id: getN.m,v 1.2 2010/03/10 18:08:15 ray Exp $
%   by Ray Zimmerman, PSERC Cornell
%   Copyright (c) 2008-2010 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/matpower/ for more info.

if nargin < 3
    N = om.(selector).N;
else
    if isfield(om.(selector).idx.N, name)
        N = om.(selector).idx.N.(name);
    else
        N = 0;
    end
end
