function [v0, vl, vu] = getv(om, name)
%GETV  Returns initial value, lower bound and upper bound for opt variables.
%   [V0, VL, VU] = GETV(OM)
%   [V0, VL, VU] = GETV(OM, NAME)
%   Returns the Returns value, lower bound and upper bound for the full
%   optimization variable vector, or for a specific named variable set.
%
%   Examples:
%       [x, xmin, xmax] = getv(om);
%       [Pg, Pmin, Pmax] = getv(om, 'Pg');
%   
%   See also OPF_MODEL.

%   MATPOWER
%   $Id: getv.m,v 1.5 2010/03/10 18:08:15 ray Exp $
%   by Ray Zimmerman, PSERC Cornell
%   Copyright (c) 2008-2010 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/matpower/ for more info.

if nargin < 2
    v0 = []; vl = []; vu = [];
    for k = 1:om.var.NS
        name = om.var.order{k};
        v0 = [ v0; om.var.data.v0.(name) ];
        vl = [ vl; om.var.data.vl.(name) ];
        vu = [ vu; om.var.data.vu.(name) ];
    end
else
    if isfield(om.var.idx.N, name)
        v0 = om.var.data.v0.(name);
        vl = om.var.data.vl.(name);
        vu = om.var.data.vu.(name);
    else
        v0 = [];
        vl = [];
        vu = [];
    end
end
